<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class EventoEstruturaController extends Crud {

    protected $dataSourceNameGet = 'getEventoEstrutura';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getEventoEstrutura(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setEventoEstrutura(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'fpa_estrutevento';
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRESTRUTEVENTO', $row)) {
                    if ($row['NRESTRUTEVENTO'] === '') {
                        $row['NRESTRUTEVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ESTRUTEVENTO', 1, 12, true); 
                    }
                } else {
                    $row['NRESTRUTEVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ESTRUTEVENTO', 1, 12, true); 
                }
                $row['NRVALORREFERENCIAM'] = !array_key_exists('NRVALORREFERENCIAM', $row) ? null : $row['NRVALORREFERENCIAM'];
                $row['NRTPAGRUPACALC'] = !array_key_exists('NRTPAGRUPACALC', $row) ? null : $row['NRTPAGRUPACALC'];
                $row['DSOBSERVACAO'] = !array_key_exists('DSOBSERVACAO', $row) ? null : $row['DSOBSERVACAO'];
                
                if($row['IDREFGERAR'] == "PARCELA"){
                    if($row['DTFIMVIGENCIA'] == null || $row['DTFIMVIGENCIA'] == ''){
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação bloqueada. Para inserir um Tipo de Referência cujo valor seja igual a Parcela, favor selecionar uma data de Início e Fim de Vigência."));
                        return;
                    }
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteEventoEstrutura(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = 'fpa_estrutevento';
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 