<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\PTO\Services\HorarioService;

class HorarioController extends Crud {

    protected $dataSourceGetHorario               = 'getHorario';
    protected $dataSourceSetHorarioM              = 'gpeHordiam_setHorarioM';
    protected $dataSourceSetHorarioH              = 'gpeHordiam_setHorarioH';
    protected $dataSourceGetIntervaloHorario      = 'getIntervaloHorario';
    protected $dataSourceSetIntervaloHorario      = 'gpeIntervalosHorario_setIntervaloHorario';
    protected $dataSourceDeleteIntervalo          = 'gpeIntervalosHorario_deleteIntervalo';
    protected $dataSourceGetIntervaloHorarioSaida = 'getSaidaHorario';
    protected $dataSourceSetIntervaloHorarioSaida = 'gpeIntervalosHorario_setSaidaIntervaloHorario';
    protected $dataSourceDeleteHorarioSaida       = 'gpeIntervalosHorario_deleteHorarioSaida';


    protected $parameterBagColumns = array('NRORG', 'DTMESCOMPETENC', 'NRHORDIAM');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HorarioService $horarioService){
        $this->parameterBag = $parameterBag;
        $this->horarioService = $horarioService;
        parent::__construct($dataSourceManager);
    }

    public function getHorario (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetHorario);
        array_pop($this->parameterBagColumns);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function getIntervaloHorario (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetIntervaloHorario);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getSaidaHorario (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetIntervaloHorarioSaida);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setHorarioM (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetHorarioM;
        try{
            foreach($request->getDataSet()->getRows() as $row) {

                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRHORDIAM'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGTRAB'], 'GPE_HORDIAM', 1, 12, true);
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
                
                $row['NRORG'] = $row['NRORGTRAB'];
            }
            parent::save($request, $response);
            $this->setHorarioH($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setHorarioH (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetHorarioH;
        try{
            foreach($request->getDataSet()->getRows() as $row) {

                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRHORDIAH'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGTRAB'], 'GPE_HORDIAH', 1, 12, true);
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
                
                $row['NRORG'] = $row['NRORGTRAB'];
                if(array_key_exists('HRINICOLETAMARCACAO', $row))
                    $row['HRINICOLETAMARCACAO'] = str_replace(':', '', $row['HRINICOLETAMARCACAO']);
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setSaidaHorario (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetIntervaloHorarioSaida;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                $row['DTULTATU'] = $row['DTATUAL'];
                $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                $row['NRORGULTATU'] = $row['NRORGTRAB'];
                $row['NRORG'] = $row['NRORGTRAB'];

                if(array_key_exists('HRPROJETARINTERVALO', $row))
                    $row['HRPROJETARINTERVALO'] = str_replace(':', '', $row['HRPROJETARINTERVALO']);

                if(!array_key_exists('DSINTERVALOPROJETADO', $row))
                    $row['DSINTERVALOPROJETADO'] = null;
                
                if(!array_key_exists('NRTPINTERVALO', $row))
                    $row['NRTPINTERVALO'] = null;
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setIntervaloHorario (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetIntervaloHorario;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRINTERVALOHORARIO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGTRAB'], 'GPE_INTERVALOSHORARIO', 1, 12, true);
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
                
                $row['NRORG'] = $row['NRORGTRAB'];
                if(array_key_exists('HRENTRADA', $row))
                    $row['HRENTRADA'] = str_replace(':', '', $row['HRENTRADA']);

                if(array_key_exists('HRSAIDA', $row))
                    $row['HRSAIDA'] = str_replace(':', '', $row['HRSAIDA']);

                if(array_key_exists('HRTOLERAANTEENTRADA', $row))
                    $row['HRTOLERAANTEENTRADA'] = str_replace(':', '', $row['HRTOLERAANTEENTRADA']);

                if(array_key_exists('HRTOLERAPOSTENTRADA', $row))
                    $row['HRTOLERAPOSTENTRADA'] = str_replace(':', '', $row['HRTOLERAPOSTENTRADA']);

                 if(array_key_exists('HRTOLERAANTESAIDA', $row))
                    $row['HRTOLERAANTESAIDA'] = str_replace(':', '', $row['HRTOLERAANTESAIDA']);

                if(array_key_exists('HRTOLERAPOSTSAIDA', $row))
                    $row['HRTOLERAPOSTSAIDA'] = str_replace(':', '', $row['HRTOLERAPOSTSAIDA']);

                if(array_key_exists('HRPROJETARINTERVALO', $row))
                    $row['HRPROJETARINTERVALO'] = str_replace(':', '', $row['HRPROJETARINTERVALO']);
                else {
                    $row['HRPROJETARINTERVALO'] = null;
                }

                if(!array_key_exists('HRINICIONUCLEO', $row))
                    $row['HRINICIONUCLEO'] = null;
                else {
                    $row['HRINICIONUCLEO'] = str_replace(':', '', $row['HRINICIONUCLEO']);
                }

                if(!array_key_exists('HRFIMNUCLEO', $row))
                    $row['HRFIMNUCLEO'] = null;
                else {
                    $row['HRFIMNUCLEO'] = str_replace(':', '', $row['HRFIMNUCLEO']);
                }

            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeHorario (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceDeleteIntervalo;
            parent::delete($request, $response);

            $this->dataSourceName = $this->dataSourceSetHorarioH;
            parent::delete($request, $response);

            $this->dataSourceName = $this->dataSourceSetHorarioM;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeSaidaHorario (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataset = $request->getDataSet();
            $dataset = $dataset->getRows();
            $dataset = $dataset[0];
            $result = $this->horarioService->removeSaidaHorario($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('removeSaidaHorario', array($result)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeIntervaloHorario (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceSetIntervaloHorario;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}